global out ~/iec/output/dise

/*******************************************************************************/
/* program dise_plot : Plot DISE panel graph, with zeroes in requested places  */
/*******************************************************************************/
cap prog drop dise_plot
prog def dise_plot
{
  syntax, [addzero(string) fit ylabel(passthru) ]
  preserve
    qui {
  capdrop new new_n b se b_up b_down pretrend
  
  /* identify potential RT vars */
  local rt_min 30
  local rt_max 10
  forval i = 10/30 {
  
    /* if this RT var does not exist, skip */
    cap if _se[rt`i'] == 0 continue
    if _rc continue

    /* update locals */
    local rt_min = min(`rt_min', `i')
    local rt_max = max(`rt_max', `i')
  }

  local num_rt = `rt_max' - `rt_min'

  /* create new observations for these betas */
  count
  local N `r(N)'
  local n_plus = `N' + `num_rt' + 1
  gen new = 0
  set obs `n_plus'
  replace new = 1 if mi(new)
  gen new_n = _n - `N' if new == 1

  /* fill in ses and betas */
  gen b = .
  gen se = .
  local c = 1

  forval i = `rt_min'/`rt_max' {
    cap replace b = _b[rt`i'] if `c' == new_n
    /* return code for graphs that are missing period -1, i.e. rt=19 */
    if _rc {
        replace b = 0 if `c' == new_n
    }
    cap replace se = _se[rt`i'] if `c' == new_n
    if _rc {
        replace b = 0 if `c' == new_n
    }
    replace relative_time = `i' - 20 if `c' == new_n
    local c = `c' + 1
  }

  /* add a point if requested */
  if !mi("`addzero'") {
      count
      local np1 = `r(N)' + 1
      set obs `np1'
      replace b = 0 if _n == `np1'
      replace se = 0 if _n == `np1'
      replace relative_time = `addzero' if _n == `np1'
      replace new = 1 if _n == `np1'
  }

  /* if "fit" is specified, fit a line from -max to -2 */
  if !mi("`fit'") {
      reg b relative_time if relative_time <= -2 & new == 1
      predict pretrend
      local pretrend (line pretrend relative_time if new == 1, lpattern(-) lcolor("128 128 128"))
  }
  
  /* generate coefplot */
  gen b_up = b + 1.96 * se
  gen b_down = b - 1.96 * se
  twoway `pretrend' (rcap b_up b_down relative_time if new) (scatter b relative_time if new, color(black) msize(vsmall)) ///
      , `ylabel' xlabel(-5(1)5) yline(0, lcolor(gs8) lpattern(-)) xline(0, lcolor(gs8) lpattern(-)) xtitle("Time Since New Road") ytitle("Log Middle School Enrollment") ///
          legend(off)

  drop if new == 1
  capdrop new new_n b se b_up b_down pretrend
}
  restore
}
end
/* *********** END program dise_plot ***************************************** */

/**************************************/
/* based on borusyak and jaravel 2017 */
/**************************************/
/*
   1. F-test for non-linear pretrends.
   2. Panel graph.
   3. Reweight all estimates in exposure regression.

*/
/************************************/
/* jacobson, lalonde and sullivan, new DISE data */
use $dise/dise_table_data_new_short, clear

/* create relative time, and add 20 so all numbers are positive. Treatment year = 20 */
gen relative_time = year - comp_year
replace relative_time = relative_time + 20

/* create relative time dummy variables and label them */
sum relative_time
forval i = `r(min)'/`r(max)' {
  local im20 = `i' - 20
  gen rt`i' = relative_time == `i'
  label var rt`i' "`im20'"
}

/* reset relative time to be centered at zero */
replace relative_time = relative_time - 20

/* regroup */
regroup pc01_state_id year
regroup pc01_state_id pc01_district_id year
regroup pc01_state_id pc01_village_id
regroup pc01_state_id pc01_district_id

/* drop unused variables */
capdrop mi_enr_total appear-elec_gain_share dise_match_source-ec98_emp_all comp_date-award_date pc11_state_id-pc11_village_id ln_enr_g-ln_enr_p sc_share-comp_month award* diff_award mp-ut num_enr*

/**********************************************/
/* save panel data prepped for borusyak graph */
/**********************************************/
save $tmp/borusyak, replace
use $tmp/borusyak, clear

keep if !mi(comp_year) & all_comp == 1

/* pool rt < -4 and rt > 4 */
gen    rt16_back = (relative_time <= -4) if !mi(relative_time)
gen rt24_forward = (relative_time >= 4)  if !mi(relative_time)

gen    rt15_back = (relative_time <= -5) if !mi(relative_time)
gen rt25_forward = (relative_time >= 5)  if !mi(relative_time)

gen    rt14_back = (relative_time <= -6) if !mi(relative_time)
gen rt26_forward = (relative_time >= 6)  if !mi(relative_time)

gen    rt13_back = (relative_time <= -7) if !mi(relative_time)
gen rt27_forward = (relative_time >= 7)  if !mi(relative_time)

drop if relative_time == -13

save $tmp/borusyak, replace
set scheme simplescheme

/* main spec */
reghdfe ln_enr_up treatment_comp , absorb(vgroup sygroup) $cluster

/* Generate pooled tails panel graph */
replace rt25 = rt25_forward
reghdfe ln_enr_up rt16-rt18 rt20-rt24 rt25, absorb(sygroup vgroup)   cluster(vgroup)
dise_plot, addzero(-5)
graphout panel_graph_new, large pdf

/* run F-test on pretrends */
test rt16=rt17=rt18=0

