use $dise/dise_village_new_clean, clear

/* define continuous dise date as september 30 of the dise year */
gen dise_date = date(string(year)+"/9/30", "YMD")
format dise_date %d

/* reset completion year based on DISE calendar year and exact build date */
drop comp_year
foreach v in comp award {

  /* format dates appropriately */
  format `v'_date %d
  
  /*  This would take 01jan2001 and give me jan */
  gen `v'_month = month(`v'_date)
  
  /*  This would take 01jan2001 and give me 2001 */
  gen `v'_year = year(`v'_date)
  
  /* adjust years for september, october, november, december */
  /* presumption is Dise measures on september 30 / october 1 */
  foreach m in 10 11 12 {
    replace `v'_year = `v'_year + 1 if `v'_month == `m'
  }
}

/* verify sanction year makes sense */
assert sanc_year <= comp_year if !mi(comp_year)

/* drop roads with implausibly early build dates */
drop if year(comp_date) < 2001 | year(award_date) < 2001

/* flag places that took unreasonably long to build. 99th percentile is 4 years, 95th is 2.8  */
gen diff_award_completion = (comp_date - award_date) / 365
replace bad = 1 if (diff_award_completion > 4 | diff_award_completion < 0.1) & !mi(diff_award_completion)

/* create indicator for unbalanced regression with all completed roads */
gen unbal = inrange(comp_year, 2002, 2015)

/* create indicators for major states with a lot of roads built */
gen mp = pc01_state_name == "madhya pradesh"
gen rj = pc01_state_name == "rajasthan"
gen up = pc01_state_name == "uttar pradesh"
gen cg = pc01_state_name == "chhattisgarh"
gen or = pc01_state_name == "orissa"
gen ap = pc01_state_name == "andhra pradesh"
gen mh = pc01_state_name == "maharashtra"
gen jh = pc01_state_name == "jharkhand"
gen as = pc01_state_name == "assam"
gen gj = pc01_state_name == "gujarat"
gen ut = pc01_state_name == "uttarakhand"

/* create road treatment indicator */
gen treatment_comp = dise_date > comp_date if !mi(dise_date)

/* regroup */
drop vgroup
egen vgroup = group(pc01_state_id pc01_village_id)

/* SAVE DISE PANEL DATASET */
sort vgroup year
save $dise/dise_working_new, replace

/*************************/
/* prepare panel dataset */
/*************************/
use $dise/dise_working_new, clear

/* keep places with no roads at baseline */
keep if pc01_vd_app_pr == 0 | con00 == 0

/* drop places that got upgrades, not new roads */
drop if road_new == 0

/* drop severe and bad */
drop if severe == 1 | bad == 1

/* set balanced panel from 2002 */
bys vgroup: egen num_enr_up =           count(enr_up) if year != 2001
bys vgroup: egen num_enr_up_post_2005 = count(enr_up) if year >= 2005

gen all_comp      = num_enr_up == 14
gen all_comp_2005 = num_enr_up_post_2005 == 11

sort vgroup year

/* regroup stuff */
regroup pc01_state_id pc01_village_id
replace vgroup = svgroup
regroup pc01_state_id year
regroup pc01_state_id pc01_district_id year
regroup pc01_state_id pc01_district_id

/* bring in BPL data and create district-level poverty measure */
merge m:1 pc01_state_id pc01_village_id using ~/iec1/frozen_data/dise-replication/dise_bpl, keep(master match) nogen
ren bpl_asset0 village_poor

/* bring in NSS data */
merge m:1 pc01_state_id pc01_district_id using ~/iec2/dise/nss_round_55_district, nogen keep(match) keepusing(unskilled_wage_gap unskilled_wage_gap_male ret_gap)

/* set above/below median NSS vars of interest */
gen nss_sample = !mi(comp_year) & all_comp == 1 & !mi(ln_enr_up) & !mi(treatment_comp) & !mi(unskilled_wage_gap) & !mi(ret_gap) & !mi(vgroup)

foreach v in ret_gap unskilled_wage_gap unskilled_wage_gap_male village_poor {
  sum `v' if nss_sample == 1, d
  gen `v'_high = `v' > `r(p50)' if !mi(`v')
}
drop nss_sample

/* create interaction variables */
gen t_wg = unskilled_wage_gap_high * treatment_comp
gen t_poor = village_poor_high * treatment_comp
gen t_ret = ret_gap_high * treatment_comp

/* create fixed effects for each subgroup */
egen inter_fe = group(sygroup unskilled_wage_gap_high ret_gap_high village_poor_high)

save $tmp/dise_table_data_new, replace

/* save short version of this dataset */
drop vilcd *name*
gen litrate_p01 = pc01_pca_p_lit / pc01_pca_tot_p
ren pc01_pca_tot_p pop
ren pc01_vd_app_pr app_pr
ren pc01_vd_edu_fac edu
ren pc01_vd_p_sch p_sch
ren pc01_vd_dist_town dist
ren pc01_pca_area pca_area
ren pc01_vd_area vd_area
ren pc01_pca_p_06 p06
label var p06 "Population age 0-6"
drop pc01_pca* pc01_vd* pc11_pca*
drop if severe == 1
drop pmgsy_state_id _merge *growth ln_pop01 severe 
ren edu pc01_vd_edu_fac
ren dist pc01_vd_dist_town
ren pop pc01_pca_tot_p
ren p_sch pc01_vd_p_sch
saveold $dise/dise_table_data_new_short, replace version(12)

/*******************/
/* Prep RD dataset */
/*******************/

/* open clean DISE dataset */
use $dise/dise_working_new, clear

drop if severe == 1

drop vilcd *name*
ren pc01_vd_edu_fac edu
ren pc01_pca_tot_p pop
ren pc01_vd_app_pr app_pr
ren pc01_vd_power_supl power

drop pc01_pca* pc01_vd* pc11_pca*
drop lati-longi pmgsy_state_id _merge *growth ln_pop01 severe
ren edu pc01_vd_edu_fac 
ren pop pc01_pca_tot_p
ren power pc01_vd_power_supl

/* create all RD variables */
/* create RD high/low and bandwidth dummies */
foreach i in 500 1000 {
  gen rd_band_0_`i' = inrange(pc01_pca_tot_p, `i' - 25, `i' + 25)
  gen rd_band_1_`i' = inrange(pc01_pca_tot_p, `i' - 50, `i' + 50)
  gen rd_band_2_`i' = inrange(pc01_pca_tot_p, `i' - 100, `i' + 100)
  gen rd_band_3_`i' = inrange(pc01_pca_tot_p, `i' - 150, `i' + 150)
  gen rd_band_4_`i' = inrange(pc01_pca_tot_p, `i' - 250, `i' + 250)

  /* RD running variables */
  gen left`i' = pc01_pca_tot_p - `i'
  gen right`i' = pc01_pca_tot_p - `i'
  replace left`i' = 0  if pc01_pca_tot_p >= `i' & !mi(pc01_pca_tot_p)
  replace right`i' = 0 if pc01_pca_tot_p < `i' & !mi(pc01_pca_tot_p)

  replace left`i' = . if !inrange(pc01_pca_tot_p, `i' - 250, `i' + 250)
  replace right`i' = . if !inrange(pc01_pca_tot_p, `i' - 250, `i' + 250)

  /* create treatment indicator */
  gen t`i' = pc01_pca_tot_p >= `i' if !mi(pc01_pca_tot_p)
}

/* generate high group dummy */
gen high = 1 if inrange(pc01_pca_tot_p, 750, 1350)
replace high = 0 if inrange(pc01_pca_tot_p, 250, 750)

/* create fixed effects interacted with high group */
egen state_high_fe = group(high pc01_state_id )
egen dist_high_fe = group(high pc01_state_id pc01_district_id )
egen subd_high_fe = group(high pc01_state_id pc01_district_id pc01_subdistrict_id )

/* create high / low dummy vars for states where we have sample */
foreach state in mp rj up cg or ap mh jh as gj ut {
  gen `state'_l = `state' & inrange(pc01_pca_tot_p, 250, 750)
  gen `state'_h = `state' & inrange(pc01_pca_tot_p, 750, 1250)
}

/* create pooled version of above 500 and 1000 variables */
foreach v in left right t rd_band_0_ rd_band_1_ rd_band_2_ rd_band_3_ rd_band_4_ {
  gen `v' = `v'500 if inrange(pc01_pca_tot_p, 250, 750)
  replace `v' = `v'1000 if inrange(pc01_pca_tot_p, 750, 1250)
}

/* create rescaled population variable */
gen pop_rd = left + right
label var pop_rd "standardized population around cutoff"

/* create binscatter quantile vars for stacked RD (pop_rd) */
foreach i in 10 25 50 100 {
  egen xq`i' = cut(pop_rd),  at(-300(`i')300)
  replace xq`i' = xq`i' + 301

  /* binscatter quantiles for full range population */
  egen pq`i' = cut(pc01_pca_tot_p), at(100(`i')1500)
}

/* generate road treatment dummies */
forval y = 2005/2014 {
  gen r`y' = comp_year < `y'
  gen a`y' = award_year <= `y'
}

/* get/build external control variables */
ext_get, src(vd01) get(pc01_vd_m_sch pc01_vd_p_sch)
ext_get, src(pca01r) get(pc01_pca_p_lit)
ext_get, src(pca11r) get(pc11_pca_tot_p)
gen pc01_lit_rate = pc01_pca_p_lit / pc01_pca_tot_p

/* generate log employment rate */
gen ln_ec98_emp_all = ln(ec98_emp_all + 1)

/* replace control vars with district means if they are missing */
foreach v in pc01_vd_edu_fac sc_share irr_share pc01_vd_power_supl ln_ec98_emp_all pc01_vd_m_sch pc01_vd_p_sch pc01_lit_rate {
  bys pc01_state_id pc01_district_id: egen tmp = mean(`v')
  replace `v' = tmp if mi(`v')
  drop tmp
}

/* save RD working data */
save $tmp/rd_working_new, replace
use $tmp/rd_working_new, clear

/***************************/
/* limit to core RD sample */
/***************************/

/* restrict sample for RD specification */
keep if inrange(pc01_pca_tot_p, 100, 1500)
drop if road_new == 0 | bad == 1
keep if con00 == 0 | app_pr == 0
drop if pop_rd == 250

/* drop impossibly high early populations or primary enrollments -- 99th percentile */
drop if enr_over_pop_2005 > .6

/* define got road indicator */
gen got_road = comp_date < dise_date if !mi(dise_date)

/* require at least one road in the district */
group pc01_state_id pc01_district_id
bys sdgroup: egen num_roads = total(got_road)
keep if num_roads > 0

/* rescale population so estimates make sense */
replace left = left / 1000
replace right = right / 1000

/* create annualized growth from 2002 to 2011 */
regroup pc01_state_id pc01_village_id 

capdrop ln_enr_up_2002 ln_enr_p_2002 
foreach v in p up {
  gen tmp = ln_enr_`v' if year == 2002
  bys svgroup: egen ln_enr_`v'_2002 = max(tmp)
  drop tmp
}

/* set IK bandwidth */
gen band = inrange(pop_rd, -180, 180)

/* limit to villages with some positive primary enrollment */
bys vgroup: egen max_p = max(ln_enr_p)
drop if max_p == 0
drop max_p

/* drop jharkhand, RD appears around 2013 so most "treated" places not actually treated in DISE years -- neither a placebo nor a treatment */
drop if jh

/* mark states / thresholds where rule was followed */
gen state_sample = rj_l | mp_l | mp_h | cg_l | cg_h | or_l | gj_l

save $tmp/rd_incl_placebo, replace

keep if state_sample
drop state_sample

save $tmp/dise_rd_table_data_new, replace
